import vobject
from typing import Optional

from . import activities
from . import data_types
from .garmin_api import GarminAPI


def to_ical(login_data: data_types.LoginData, limit: int,
            activity_type: Optional[data_types.ActivityType],
            measurement_system: Optional[data_types.MeasurementSystem]) -> str:
    with GarminAPI(login_data) as api:
        activities_data = api.activites(limit, activity_type)

        cal = vobject.iCalendar()
        for activity_data in activities_data:
            activity = activities.get_activity(
                activity_data, api, measurement_system=measurement_system)
            event = vobject.newFromBehavior("vevent")

            event.add("uid").value = activity.ical_uid
            event.add("summary").value = activity.ical_summary
            event.add("dtstart").value = activity.ical_dtstart
            event.add("dtend").value = activity.ical_dtend
            event.add("description").value = activity.detail_link

            cal.add(event)

    return cal.serialize()



import vobject
from datetime import datetime, timedelta, tzinfo
from typing import Optional

from . import format_utils
from .garmin_api import GarminAPI, GARMIN_WEB_BASE_URI
from .data_types import ActivityType, ActivityData, ActivityTypeData, \
    MeasurementSystem


class Activity:

    def __init__(self, data: ActivityData, garmin_api: GarminAPI,
                 measurement_system: Optional[MeasurementSystem]):
        self._data = data
        self._garmin_api = garmin_api
        self._measurement_system = measurement_system
        if not self._measurement_system:
            self._measurement_system = MeasurementSystem.METRIC
        self._tzinfo = get_tzinfo_from_garmin_id(
            self._data["timeZoneId"], garmin_api)

    @property
    def name(self) -> str:
        return str(self._data.get("activityName", "Other"))

    @property
    def duration(self) -> str:
        duration = self._data.get("duration")

        if duration is None:
            return "---"

        return format_utils.time(duration)

    @property
    def distance(self) -> str:
        distance = self._data.get("distance")

        if distance is None:
            return "---"

        if self._measurement_system is MeasurementSystem.METRIC:
            return format_utils.distance_kilometers(distance)
        else:
            return format_utils.distance_miles(distance)

    @property
    def average_speed(self) -> str:
        speed = self._data.get("averageSpeed")

        if speed is None:
            return "---"

        if self._measurement_system is MeasurementSystem.METRIC:
            return format_utils.speed_kmph(speed)
        else:
            return format_utils.speed_mph(speed)

    @property
    def detail_link(self) -> str:
        return f"{GARMIN_WEB_BASE_URI}/activity/{self._data['activityId']}"

    @property
    def ical_uid(self) -> str:
        return (
            f"garmin-activity-"
            f"{self._data['ownerId']}-{self._data['activityId']}")

    @property
    def ical_summary(self) -> str:
        return f"{self.name} ({self.distance}, {self.duration})"

    @property
    def ical_dtstart(self) -> datetime:
        start_time = datetime.strptime(
            self._data["startTimeLocal"], "%Y-%m-%d %H:%M:%S")
        if self._tzinfo:
            start_time = start_time.replace(tzinfo=self._tzinfo)
        return start_time

    @property
    def ical_dtend(self) -> datetime:
        duration = 0
        if self._data["duration"] is not None:
            duration = round(self._data["duration"])
        return self.ical_dtstart + timedelta(seconds=duration)


class RunningActivity(Activity):

    @property
    def average_speed(self):
        speed = self._data.get("averageSpeed")

        if speed is None:
            return "---"

        if self._measurement_system is MeasurementSystem.METRIC:
            return format_utils.speed_minutes_per_kilometer(speed)
        else:
            return format_utils.speed_minutes_per_mile(speed)

    @property
    def ical_summary(self):
        return f"{self.name} ({self.distance}, {self.average_speed})"


class CyclingActivity(Activity):

    @property
    def ical_summary(self):
        return f"{self.name} ({self.distance}, {self.average_speed})"


class SwimmingActivity(Activity):

    @property
    def distance(self) -> str:
        dist = self._data.get("distance")
        return "---" if dist is None else format_utils.distance_metres(dist)

    @property
    def average_speed(self):
        speed = self._data.get("averageSpeed")

        if speed is None:
            return "---"

        return format_utils.speed_minutes_per_100_metres(speed)

    @property
    def ical_summary(self):
        return f"{self.name} ({self.distance}, {self.average_speed})"


class MultisportActivity(Activity):

    @property
    def ical_summary(self):
        return f"{self.name} ({self.duration}, {self.distance})"


class FitnessActivity(Activity):

    @property
    def ical_summary(self):
        return f"{self.name} ({self.duration})"


def get_activity(data: ActivityData, garmin_api: GarminAPI,
                 measurement_system: Optional[MeasurementSystem]) -> Activity:
    activity_type = get_activity_type(data, garmin_api)

    if activity_type is ActivityType.RUNNING:
        return RunningActivity(data, garmin_api, measurement_system)

    if activity_type is ActivityType.CYCLING:
        return CyclingActivity(data, garmin_api, measurement_system)

    if activity_type is ActivityType.SWIMMING:
        return SwimmingActivity(data, garmin_api, measurement_system)

    if activity_type is ActivityType.MULTISPORT:
        return MultisportActivity(data, garmin_api, measurement_system)

    if activity_type is ActivityType.FITNESS:
        return FitnessActivity(data, garmin_api, measurement_system)

    return Activity(data, garmin_api, measurement_system)


def get_activity_type(data: ActivityData,
                      garmin_api: GarminAPI) -> ActivityType:
    all_types = garmin_api.activity_types()
    type_data = data["activityType"]

    def is_known(type_data: ActivityTypeData):
        try:
            ActivityType(type_data["typeKey"])
            return True
        except ValueError:
            return False

    while not is_known(type_data):
        try:
            type_data = next(filter(
                lambda t: t["typeId"] == type_data["parentTypeId"], all_types))
        except StopIteration:
            break

    if is_known(type_data):
        return ActivityType(type_data["typeKey"])

    return ActivityType.OTHER


def get_tzinfo_from_garmin_id(
        id: int,
        garmin_api: GarminAPI) -> Optional[tzinfo]:
    timezones = garmin_api.timezones()
    match = next(filter(lambda tz: tz["unitId"] == id, timezones), None)
    return vobject.icalendar.getTzid(match["timeZone"]) if match else None



import requests
import urllib.parse
from functools import lru_cache
from typing import Optional, List, Dict, Any  # noqa

from .data_types import LoginData, ActivityType, ActivityData, \
    ActivityTypeData, TimezoneData


GARMIN_WEB_BASE_URI = "https://connect.garmin.com/modern"

LOGIN_URI = (
    f"https://sso.garmin.com/sso/signin?"
    f"{urllib.parse.urlencode({'service': GARMIN_WEB_BASE_URI})}")


class GarminAPI:

    def __init__(self, login_data: LoginData):
        self.session = self.login(login_data)

    def __enter__(self) -> "GarminAPI":
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.session.close()

    @staticmethod
    def login(login_data: LoginData) -> requests.Session:
        session = requests.Session()
        session.headers.update({
            "referer": LOGIN_URI,
            "NK": "NT",
            "User-Agent": "Mozilla/5.0 (+https://github.com/jirikuchta/garmin-ical-export)"
        })
        session.post(LOGIN_URI, login_data._asdict())
        session.get(GARMIN_WEB_BASE_URI)
        return session

    def activites(self, limit: int,
                  activity_type: Optional[ActivityType]) -> List[ActivityData]:
        return get_activites(self.session, limit, activity_type)

    def activity_types(self) -> List[ActivityTypeData]:
        return get_activity_types(self.session)

    def timezones(self) -> List[TimezoneData]:
        return get_timezones(self.session)


@lru_cache()
def get_activites(session: requests.Session, limit: int,
                  activity_type: Optional[ActivityType]) -> List[ActivityData]:
    path = "/proxy/activitylist-service/activities/search/activities"
    qs = {"limit": limit}  # type: Dict[str, Any]
    if activity_type and activity_type is not ActivityType.ALL:
        qs["activityType"] = activity_type.value
    uri = f"{GARMIN_WEB_BASE_URI}{path}?{urllib.parse.urlencode(qs)}"
    res = session.get(uri)
    res.raise_for_status()
    return res.json()


@lru_cache()
def get_activity_types(session: requests.Session) -> List[ActivityTypeData]:
    path = "/proxy/activity-service/activity/activityTypes"
    res = session.get(f"{GARMIN_WEB_BASE_URI}{path}")
    res.raise_for_status()
    return res.json()


@lru_cache()
def get_timezones(session: requests.Session) -> List[TimezoneData]:
    path = "/proxy/system-service/timezoneUnits"
    res = session.get(f"{GARMIN_WEB_BASE_URI}{path}")
    res.raise_for_status()
    return res.json()



import enum
from collections import namedtuple
from mypy_extensions import TypedDict
from typing import Optional


LoginData = namedtuple("LoginData", "username password")


class ActivityType(enum.Enum):
    RUNNING = "running"
    CYCLING = "cycling"
    SWIMMING = "swimming"
    MULTISPORT = "multi_sport"
    FITNESS = "fitness_equipment"
    HIKING = "hiking"
    WALKING = "walking"
    WINTERSPORTS = "winter_sports"
    OTHER = "other"
    ALL = "all"

    def __str__(self):
        return self.value


class ActivityTypeData(TypedDict):
    typeId: int
    typeKey: str
    parentTypeId: int


class ActivityData(TypedDict):
    activityId: int
    activityName: Optional[str]
    activityType: ActivityTypeData
    averageSpeed: Optional[float]  # m/s
    description: Optional[str]
    distance: Optional[float]  # metres
    duration: Optional[float]  # seconds
    ownerId: int
    startTimeLocal: str
    timeZoneId: int


class TimezoneData(TypedDict):
    unitId: int
    timeZone: str


class MeasurementSystem(enum.Enum):
    METRIC = "metric"
    IMPERIAL = "imperial"

    def __str__(self):
        return self.value



#!/usr/bin/env python3

import setuptools  # noqa
from distutils.core import setup


def readme():
    with open("README.md", encoding='utf-8') as f:
        return f.read()


setup(
    name="garmin-ical-export",
    version="1.0.5",
    description="Export Garmin Connect activities to iCalendar file",
    long_description=readme(),
    long_description_content_type="text/markdown",
    author="Jiri Kuchta",
    author_email="jiri.kuchta@email.cz",
    url="https://github.com/jirikuchta/garmin-ical-export",
    packages=["garminicalexport"],
    license="MIT",
    entry_points={
        "console_scripts": ["garmin-ical-export=garminicalexport.cmd:main"]
    },
    install_requires=["vobject", "requests", "mypy_extensions"],
    python_requires=">=3.6",
    include_package_data=True,
    zip_safe=False
)



def time(seconds: float) -> str:
    hours, remainder = divmod(round(seconds), 3600)
    minutes, seconds = divmod(remainder, 60)
    return f"{hours:02}:{minutes:02}:{seconds:02}"


def distance_metres(meters: float) -> str:
    return f"{round(meters)} m"


def distance_kilometers(meters: float) -> str:
    return f"{round(meters / 1000, 2)} km"


def distance_miles(meters: float) -> str:
    return f"{round((meters / 1000) * 0.62137, 2)} mi"


def speed_kmph(meters_per_second: float) -> str:
    return f"{round(meters_per_second * 3.6, 1)} km/h"


def speed_mph(meters_per_second: float) -> str:
    return f"{round((meters_per_second * 3.6) * 0.62137, 1)} mi/h"


def speed_minutes_per_100_metres(meters_per_second: float) -> str:
    try:
        minutes, seconds = divmod(100 / meters_per_second, 60)
    except ZeroDivisionError:
        minutes, seconds = (0, 0)
    return f"{int(minutes)}:{round(seconds):02} min/100m"


def speed_minutes_per_kilometer(meters_per_second: float) -> str:
    try:
        minutes, seconds = divmod(1000 / meters_per_second, 60)
    except ZeroDivisionError:
        minutes, seconds = (0, 0)
    return f"{int(minutes)}:{round(seconds):02} min/km"


def speed_minutes_per_mile(meters_per_second: float) -> str:
    try:
        minutes, seconds = divmod((1000 / 0.62137) / meters_per_second, 60)
    except ZeroDivisionError:
        minutes, seconds = (0, 0)
    return f"{int(minutes)}:{round(seconds):02} min/mi"



#!/usr/bin/env python3

import argparse

from . import to_ical
from .data_types import LoginData, ActivityType, MeasurementSystem


def parse_arguments():
    parser = argparse.ArgumentParser(
        description="Exports Garmin Connect activities to iCalendar file.")
    parser.add_argument(
        "garmin_username",
        type=str,
        metavar="login_email",
        help="your Garmin Connect login e-mail")
    parser.add_argument(
        "garmin_password",
        type=str,
        metavar="password",
        help="your Garmin Connect login password")
    parser.add_argument(
        "--activity_type",
        default="all",
        type=ActivityType,
        choices=list(ActivityType),
        help="export only specific type of activity")
    parser.add_argument(
        "--limit",
        type=int,
        default=10000,
        help="max. number of activities to export")
    parser.add_argument(
        "--measurement_system",
        default=MeasurementSystem.METRIC,
        type=MeasurementSystem,
        choices=list(MeasurementSystem),
        help="which system to use to calculate activity properties")
    parser.add_argument(
        "--target_file",
        type=str,
        help="target .ics file (prints to stdout if not passed)")

    return parser.parse_args()


def main():
    args = parse_arguments()

    login_data = LoginData(args.garmin_username, args.garmin_password)

    ical = to_ical(login_data, args.limit,
                   activity_type=args.activity_type,
                   measurement_system=args.measurement_system)

    if args.target_file:
        with open(args.target_file, "w+") as file:
            file.write(ical)
    else:
        print(ical)


if __name__ == "__main__":
    main()
